@@ -3,9 +3,10 @@
 from __future__ import annotations
 
 from math import ceil
-from typing import Callable, Hashable
+from typing import Any, Callable, Hashable, Sequence
 
-from rapidfuzz._utils import is_none
+from rapidfuzz._common_py import conv_sequences
+from rapidfuzz._utils import ScorerFlag, add_scorer_attrs, is_none
 from rapidfuzz.distance import ScoreAlignment
 from rapidfuzz.distance.Indel_py import (
     _block_normalized_similarity as indel_block_normalized_similarity,
@@ -14,7 +15,17 @@ from rapidfuzz.distance.Indel_py import distance as indel_distance
 from rapidfuzz.distance.Indel_py import (
     normalized_similarity as indel_normalized_similarity,
 )
-from rapidfuzz.utils_py import default_process
+
+
+def get_scorer_flags_fuzz(**_kwargs: Any) -> dict[str, Any]:
+    return {
+        "optimal_score": 100,
+        "worst_score": 0,
+        "flags": ScorerFlag.RESULT_F64 | ScorerFlag.SYMMETRIC,
+    }
+
+
+fuzz_attribute: dict[str, Callable[..., dict[str, Any]]] = {"get_scorer_flags": get_scorer_flags_fuzz}
 
 
 def _norm_distance(dist: int, lensum: int, score_cutoff: float) -> float:
@@ -23,10 +34,10 @@ def _norm_distance(dist: int, lensum: int, score_cutoff: float) -> float:
 
 
 def ratio(
-    s1: str | bytes | None,
-    s2: str | bytes | None,
+    s1: Sequence[Hashable] | None,
+    s2: Sequence[Hashable] | None,
     *,
-    processor: Callable[..., str | bytes] | None | bool = None,
+    processor: Callable[..., Sequence[Hashable]] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -34,9 +45,9 @@ def ratio(
 
     Parameters
     ----------
-    s1 : str | bytes
+    s1 : Sequence[Hashable]
         First string to compare.
-    s2 : str | bytes
+    s2 : Sequence[Hashable]
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
@@ -53,7 +64,7 @@ def ratio(
 
     See Also
     --------
-    rapidfuzz.string_metric.normalized_levenshtein : Normalized levenshtein distance
+    rapidfuzz.distance.Indel.normalized_distance : Normalized Indel distance
 
     Notes
     -----
@@ -67,23 +78,14 @@ def ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if score_cutoff is not None:
         score_cutoff /= 100
 
-    score = indel_normalized_similarity(
-        s1, s2, processor=processor, score_cutoff=score_cutoff
-    )
+    score = indel_normalized_similarity(s1, s2, processor=processor, score_cutoff=score_cutoff)
     return score * 100
 
 
-def _partial_ratio_short_needle(
-    s1: str | bytes, s2: str | bytes, score_cutoff: float
-) -> ScoreAlignment:
+def _partial_ratio_short_needle(s1: Sequence[Hashable], s2: Sequence[Hashable], score_cutoff: float) -> ScoreAlignment:
     """
     implementation of partial_ratio for needles <= 64. assumes s1 is already the
     shorter string
@@ -107,9 +109,7 @@ def _partial_ratio_short_needle(
             continue
 
         # todo cache map
-        ls_ratio = indel_block_normalized_similarity(
-            block, s1, s2[:i], score_cutoff=score_cutoff
-        )
+        ls_ratio = indel_block_normalized_similarity(block, s1, s2[:i], score_cutoff=score_cutoff)
         if ls_ratio > res.score:
             res.score = score_cutoff = ls_ratio
             res.dest_start = 0
@@ -124,9 +124,7 @@ def _partial_ratio_short_needle(
             continue
 
         # todo cache map
-        ls_ratio = indel_block_normalized_similarity(
-            block, s1, s2[i : i + len1], score_cutoff=score_cutoff
-        )
+        ls_ratio = indel_block_normalized_similarity(block, s1, s2[i : i + len1], score_cutoff=score_cutoff)
         if ls_ratio > res.score:
             res.score = score_cutoff = ls_ratio
             res.dest_start = i
@@ -141,9 +139,7 @@ def _partial_ratio_short_needle(
             continue
 
         # todo cache map
-        ls_ratio = indel_block_normalized_similarity(
-            block, s1, s2[i:], score_cutoff=score_cutoff
-        )
+        ls_ratio = indel_block_normalized_similarity(block, s1, s2[i:], score_cutoff=score_cutoff)
         if ls_ratio > res.score:
             res.score = score_cutoff = ls_ratio
             res.dest_start = i
@@ -157,10 +153,10 @@ def _partial_ratio_short_needle(
 
 
 def partial_ratio(
-    s1: str | bytes | None,
-    s2: str | bytes | None,
+    s1: Sequence[Hashable] | None,
+    s2: Sequence[Hashable] | None,
     *,
-    processor: Callable[..., str | bytes] | None | bool = None,
+    processor: Callable[..., Sequence[Hashable]] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -169,9 +165,9 @@ def partial_ratio(
 
     Parameters
     ----------
-    s1 : str | bytes
+    s1 : Sequence[Hashable]
         First string to compare.
-    s2 : str | bytes
+    s2 : Sequence[Hashable]
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
@@ -227,9 +223,7 @@ def partial_ratio(
     >>> fuzz.partial_ratio("this is a test", "this is a test!")
     100.0
     """
-    alignment = partial_ratio_alignment(
-        s1, s2, processor=processor, score_cutoff=score_cutoff
-    )
+    alignment = partial_ratio_alignment(s1, s2, processor=processor, score_cutoff=score_cutoff)
     if alignment is None:
         return 0
 
@@ -240,7 +234,7 @@ def partial_ratio_alignment(
     s1: str | bytes | None,
     s2: str | bytes | None,
     *,
-    processor: Callable[..., str | bytes] | None | bool = None,
+    processor: Callable[..., str | bytes] | None = None,
     score_cutoff: float | None = None,
 ) -> ScoreAlignment | None:
     """
@@ -283,11 +277,6 @@ def partial_ratio_alignment(
     if is_none(s1) or is_none(s2):
         return None
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -298,6 +287,7 @@ def partial_ratio_alignment(
     if not s1 and not s2:
         return ScoreAlignment(100.0, 0, 0, 0, 0)
 
+    s1, s2 = conv_sequences(s1, s2)
     if len(s1) <= len(s2):
         shorter = s1
         longer = s2
@@ -310,9 +300,7 @@ def partial_ratio_alignment(
         score_cutoff = max(score_cutoff, res.score)
         res2 = _partial_ratio_short_needle(longer, shorter, score_cutoff / 100)
         if res2.score > res.score:
-            res = ScoreAlignment(
-                res2.score, res2.dest_start, res2.dest_end, res2.src_start, res2.src_end
-            )
+            res = ScoreAlignment(res2.score, res2.dest_start, res2.dest_end, res2.src_start, res2.src_end)
 
     if res.score < score_cutoff:
         return None
@@ -320,16 +308,14 @@ def partial_ratio_alignment(
     if len(s1) <= len(s2):
         return res
 
-    return ScoreAlignment(
-        res.score, res.dest_start, res.dest_end, res.src_start, res.src_end
-    )
+    return ScoreAlignment(res.score, res.dest_start, res.dest_end, res.src_start, res.src_end)
 
 
 def token_sort_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -343,7 +329,7 @@ def token_sort_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -366,11 +352,6 @@ def token_sort_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -384,7 +365,7 @@ def token_set_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -399,7 +380,7 @@ def token_set_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -424,11 +405,6 @@ def token_set_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -490,7 +466,7 @@ def token_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -505,7 +481,7 @@ def token_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -523,11 +499,6 @@ def token_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -543,7 +514,7 @@ def partial_token_sort_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -557,7 +528,7 @@ def partial_token_sort_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -575,11 +546,6 @@ def partial_token_sort_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -593,7 +559,7 @@ def partial_token_set_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -608,7 +574,7 @@ def partial_token_set_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -626,11 +592,6 @@ def partial_token_set_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -655,7 +616,7 @@ def partial_token_ratio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -670,7 +631,7 @@ def partial_token_ratio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -688,11 +649,6 @@ def partial_token_ratio(
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -737,7 +693,7 @@ def WRatio(
     s1: str | None,
     s2: str | None,
     *,
-    processor: Callable[..., str] | None | bool = default_process,
+    processor: Callable[..., str] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
@@ -751,7 +707,7 @@ def WRatio(
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -771,11 +727,6 @@ def WRatio(
 
     UNBASE_SCALE = 0.95
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -797,46 +748,42 @@ def WRatio(
         score_cutoff = max(score_cutoff, end_ratio) / UNBASE_SCALE
         return max(
             end_ratio,
-            token_ratio(s1, s2, score_cutoff=score_cutoff, processor=None)
-            * UNBASE_SCALE,
+            token_ratio(s1, s2, score_cutoff=score_cutoff, processor=None) * UNBASE_SCALE,
         )
 
     PARTIAL_SCALE = 0.9 if len_ratio < 8.0 else 0.6
     score_cutoff = max(score_cutoff, end_ratio) / PARTIAL_SCALE
-    end_ratio = max(
-        end_ratio, partial_ratio(s1, s2, score_cutoff=score_cutoff) * PARTIAL_SCALE
-    )
+    end_ratio = max(end_ratio, partial_ratio(s1, s2, score_cutoff=score_cutoff) * PARTIAL_SCALE)
 
     score_cutoff = max(score_cutoff, end_ratio) / UNBASE_SCALE
     return max(
         end_ratio,
-        partial_token_ratio(s1, s2, score_cutoff=score_cutoff, processor=None)
-        * UNBASE_SCALE
-        * PARTIAL_SCALE,
+        partial_token_ratio(s1, s2, score_cutoff=score_cutoff, processor=None) * UNBASE_SCALE * PARTIAL_SCALE,
     )
 
 
 def QRatio(
-    s1: str | bytes | None,
-    s2: str | bytes | None,
+    s1: Sequence[Hashable] | None,
+    s2: Sequence[Hashable] | None,
     *,
-    processor: Callable[..., str | bytes] | None | bool = default_process,
+    processor: Callable[..., Sequence[Hashable]] | None = None,
     score_cutoff: float | None = None,
 ) -> float:
     """
     Calculates a quick ratio between two strings using fuzz.ratio.
-    The only difference to fuzz.ratio is, that this preprocesses
-    the strings by default.
+
+    Since v3.0 this behaves similar to fuzz.ratio with the exception that this
+    returns 0 when comparing two empty strings
 
     Parameters
     ----------
-    s1 : str | bytes
+    s1 : Sequence[Hashable]
         First string to compare.
-    s2 : str | bytes
+    s2 : Sequence[Hashable]
         Second string to compare.
     processor: callable, optional
         Optional callable that is used to preprocess the strings before
-        comparing them. Default is ``utils.default_process``.
+        comparing them. Default is None, which deactivates this behaviour.
     score_cutoff : float, optional
         Optional argument for a score threshold as a float between 0 and 100.
         For ratio < score_cutoff 0 is returned instead. Default is 0,
@@ -849,17 +796,12 @@ def QRatio(
 
     Examples
     --------
-    >>> fuzz.QRatio("this is a test", "THIS is a test!")
-    100.0
+    >>> fuzz.QRatio("this is a test", "this is a test!")
+    96.55171966552734
     """
     if is_none(s1) or is_none(s2):
         return 0
 
-    if processor is True:
-        processor = default_process
-    elif processor is False:
-        processor = None
-
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
@@ -869,3 +811,15 @@ def QRatio(
         return 0
 
     return ratio(s1, s2, score_cutoff=score_cutoff)
+
+
+add_scorer_attrs(ratio, fuzz_attribute)
+add_scorer_attrs(partial_ratio, fuzz_attribute)
+add_scorer_attrs(token_sort_ratio, fuzz_attribute)
+add_scorer_attrs(token_set_ratio, fuzz_attribute)
+add_scorer_attrs(token_ratio, fuzz_attribute)
+add_scorer_attrs(partial_token_sort_ratio, fuzz_attribute)
+add_scorer_attrs(partial_token_set_ratio, fuzz_attribute)
+add_scorer_attrs(partial_token_ratio, fuzz_attribute)
+add_scorer_attrs(WRatio, fuzz_attribute)
+add_scorer_attrs(QRatio, fuzz_attribute)
